## Holger Brandt
## 20170601

## packages
library(lavaan)
library(mvtnorm)
library(scatterplot3d)
library(semPlot)

##################################################
##################################################
# exercise 1. constrained product indicator approach
##################################################
##################################################

dat.kj <- read.table("../data/data_kj.dat", header = TRUE)

# center data
dat.kj$v1 <- dat.kj$v1-mean(dat.kj$v1)
dat.kj$v2 <- dat.kj$v2-mean(dat.kj$v2)
dat.kj$s1 <- dat.kj$s1-mean(dat.kj$s1)
dat.kj$s2 <- dat.kj$s2-mean(dat.kj$s2)

# build pi's
dat.kj$vs1 <- dat.kj$v1*dat.kj$s1
dat.kj$vs2 <- dat.kj$v2*dat.kj$s2

plot(density(dat.kj$v1))
plot(density(dat.kj$vs1))

# note: we only use 2 pi's 
# because the model becomes very complex if one
# included v1*s2 and v2*s1, too
# (formulation of residual covariance constraints necessary)

# data set including pi's
head(dat.kj)

#########################################
# the model syntax for constrained pi
#########################################
model.cpi <- '
#####################
# linear model part
#####################
# measurement model 
# (with two indicators, 2nd loading is fixed to 1 for identification)
xi1  =~ v1+1*v2
xi2  =~ s1+1*s2
eta  =~ c1+1*c2
# intercepts are all zero (centered data)
v1 ~ 0*1
v2 ~ 0*1
s1 ~ 0*1
s2 ~ 0*1
c1 ~ 0*1
c2 ~ 0*1
# residual variances
v1 ~~ td11*v1
v2 ~~ td22*v2
s1 ~~ td33*s1
s2 ~~ td44*s2
c1 ~~ te11*c1
c2 ~~ te22*c2
# structural model
eta ~ 1+ xi1 + xi2 + gamma3*xi1xi2 
# latent means
xi1 ~ 0*1
xi2 ~ 0*1
# latent variances and covariances
xi1 ~~ phi11*xi1
xi1 ~~ phi12*xi2
xi2 ~~ phi22*xi2
#####################
# nonlinear model part
#####################
# measurement model
# constrained factor loadings (all equal to 1)
xi1xi2 =~ vs1+1*vs2
# constrained intercepts (all zero)
vs1 ~ 0*1
vs2 ~ 0*1
# constrained variances and covariances
xi1 ~~ 0*xi1xi2
xi2 ~~ 0*xi1xi2
xi1xi2 ~~ phi13*xi1xi2
phi13 == phi11*phi22+phi12^2
# constraints latent means
xi1xi2 ~ phi12*1
# constrained residual variances
vs1 ~~ td55*vs1
vs2 ~~ td66*vs2
td55 == phi11*td33+phi22*td11+td11*td33
td66 == phi11*td44+phi22*td22+td22*td44
'
#########################################

# estimate the model
sem.cpi <- sem(model.cpi,dat=dat.kj)
sem.cpi <- sem(model.cpi,dat=dat.kj,estimator="mlr")
summary(sem.cpi)

# path diagram if wanted
semPaths(sem.cpi,title = FALSE, curvePivot = TRUE)

# comparison model 
sem.cpi2 <- sem(model.cpi,dat=dat.kj,estimator="mlr",
                constraints = "gamma3==0")
anova(sem.cpi,sem.cpi2)


######################################################################
# note: 
# 1. constraints are only valid for normal and centered data 
######################################################################
# 2. model fit investigation is complicated (more about that later)
#    here just some indication of the problem:

model.lin <- '
xi1  =~ v1+1*v2
xi2  =~ s1+1*s2
eta  =~ c1+1*c2
eta ~ 1+ xi1 + xi2
v1 ~ 0*1
v2 ~ 0*1
s1 ~ 0*1
s2 ~ 0*1
c1 ~ 0*1
c2 ~ 0*1
'

sem.lin <- sem(model.lin,dat=dat.kj)
summary(sem.lin)

# comparison of fit between linear and interaction model
round(cbind(fitmeasures(sem.lin),fitmeasures(sem.cpi)),2)
# the missing interaction effect does not show up!
######################################################################
# 3. the model is parsimonious because it has only 1 parameter more (interaction effect)
# 4. do not use the standardized solution (again, more about that later)


##################################################
##################################################
# exercise 2. unconstrained product indicator approach
##################################################
##################################################

######################################################################
# the unconstrained approach for two indicator and dat.kj
######################################################################
# specify the syntax (use the one above as a starting point)
model.upi <- '
#...
'


# run the model

# results

# compare the model sem.upi and sem.cpi





##################################################
##################################################
# exercise 3. unconstrained pi approach for advanced example
##################################################
##################################################

##################################################
# This data set is a subset of Australian pupils 
# in the PISA data set 2009 (OECD, 2012)
##################################################
dat <- read.table("../data/pisa_online1.dat",header=T,na.strings = "999")

head(dat)

# on1...on3: online activities (reading emails etc) Q26
# att1...att3: reading attitude (like reading etc)  Q24
# read1...read3: reading skill (from reading test)

# delete missings for now
dat <- na.omit(dat)

##################################################
# PART I: data inspection
# this part allow to get familiar with the data set
##################################################
library(moments)
library(MVN)
library(MASS)

# observed variables
online   <- apply(dat[,1:3],1,mean)
attitude <- apply(dat[,4:6],1,mean)
reading  <- apply(dat[,7:9],1,mean)

# univariate histograms
par(mfrow=c(1,3))
hist(online,freq=F)
curve(dnorm(x=x,mean=mean(online),sd=sd(online)),
      from=min(online),to=max(online),add=T)

hist(attitude,freq=F)
curve(dnorm(x=x,mean=mean(attitude),sd=sd(attitude)),
      from=min(attitude),to=max(attitude),add=T)

hist(reading,freq=F)
curve(dnorm(x=x,mean=mean(reading),sd=sd(reading)),from=min(reading),to=max(reading),add=T)

# bivariate scatterplot with loess approximation
par(mfrow=c(1,2))
scatter.smooth(online,reading)
scatter.smooth(attitude,reading)

# descriptive statistics
round(apply(dat,2,mean),3)
round(apply(dat,2,var),3)
round(apply(dat,2,skewness),3)
round(apply(dat,2,kurtosis),3)-3

# multivariate nonnormality
mardiaTest(dat[,1:6],qqplot=T) #iv
mardiaTest(dat[,7:8],qqplot=T) #dv

# predictor variables: bivariate distribution
plotmv1 <- kde2d(online, attitude, n = 50)
image(plotmv1)
contour(plotmv1, add = T)


##################################################
# PART II: data analysis with nonlinear models
##################################################

# manifest data and regression indicate quadratic effect
summary(lm(reading~ attitude*online+I(attitude^2)+I(online^2)))

###########################
# data preparation for SEM
# 1. center
# 2. pi's: use pairs (x1x4,x2x5,x3x6 for interaction...) to reduce redundant information
###########################
#head(dat)


###################################################
# model: interaction + quadratic model
# -- start with the model syntax from above for the unconstrained pi
# -- extend the measurement model (you will have a total of 6 latent variables, including eta)
# -- constraint the intercepts (because we centered the data)
# -- constraint unnecessary covariances for the predictors
# -- set the means for xi1 and xi2 to zero (but not those of the latent product terms) 
# -- important: include necessary residual covariances
###################################################
model.upi2 <- '
#...
'

# run model

# model comparison
# important: use same information (i.e. pi indicators)
# fix the regression coefficient to zero, 
# e.g., by including a parameter name in the syntax and use of constraint="..." here


# path diagram if wanted
semPaths(sem.upi2,title = FALSE, curvePivot = TRUE)




########################################################
# this section provides some possibilities 
# to illustrate and interpret the results
########################################################
# extract unstandardized coefficient
gamma.upi  <- coef(sem.upi3)[paste0("ga",1:5)]
# standardized
std.upi    <- standardizedsolution(sem.upi3)
gammas.upi <- std.upi[std.upi[,1]=="eta"&std.upi[,2]=="~",4]

# correct standardization
phi.upi <- coef(sem.upi3)[c("phi11","phi22","xi1xi2~~xi1xi2","xi1q~~xi1q","xi2q~~xi2q")]
gammas.upi[3] <- gammas.upi[3]*sqrt((phi.upi[1]*phi.upi[2])/phi.upi[3])
gammas.upi[4] <- gammas.upi[4]*sqrt((phi.upi[1]^2)/phi.upi[4])
gammas.upi[5] <- gammas.upi[5]*sqrt((phi.upi[2]^2)/phi.upi[5])


# calculate expected value for dependent variable
yhat <- function(x,y){
  # function returns expected value
  # input: x and y == two predictor variables
  # gamma.s: standardized coefficients
  yhat <- gammas.upi[1]*x+gammas.upi[2]*y+
    gammas.upi[3]*x*y+gammas.upi[4]*x^2+gammas.upi[5]*y^2
  return(yhat)
}

#################################
# simple slopes for fixed attitude
#################################
# empty plot
par(mfrow=c(1,1))
plot(0,0,col="white",xlim=c(-3,3),ylim=c(-2.8,2.8),
     ylab="Predicted Reading Skills",
     xlab="Online Activities")
# add curves for different fixed values of a predictor 
#variable
curve(yhat(x=x,y=-3),add=T,lty="dotted")
curve(yhat(x=x,y= 0),add=T)
curve(yhat(x=x,y= 3),add=T,lty="dashed")
# add a legend
legend(x=-3,y=2,
       c("Att=+3",
         "Att=0",
         "Att=-3"),
       lty=c("dashed","solid","dotted"))


#################################
# simple slopes for fixed online activities
#################################
plot(0,0,col="white",xlim=c(-3,3),ylim=c(-2.8,2.8),
     ylab="Predicted Reading Skills",
     xlab="Reading Attitude")
# add curves for different fixed values of a predictor 
#variable
curve(yhat(x=-3,y=x),add=T,lty="dotted")
curve(yhat(x=0,y= x),add=T)
curve(yhat(x=3,y= x),add=T,lty="dashed")
# add a legend
legend(x=-3,y=2,
       c("Onl=+3",
         "Onl=0",
         "Onl=-3"),
       lty=c("dashed","solid","dotted"))



#################################
# 3d plot
#################################
# limits of the predictor variables (min, max)
yr <- xr <- c(-3,3)
curve_3d <- function(yhat, 
                     x_range=xr, 
                     y_range=yr, 
                     col=1:6){
  # function produces 3d plot
  #input: function for dependent variable (yhat)
  #       ranges for predictor variables
  #       colors for graphic
  if (!require(rgl) ) {stop("load rgl")}
  #produces sequence for predictor variables
  xvec <- seq(x_range[1], x_range[2], len=50)
  yvec <- seq(y_range[1], y_range[2], len=50)
  # computes values for dependent variable
  fz   <- outer(xvec, yvec, FUN=yhat)
  open3d()
  # creates graphic with specifics such as labels
  persp3d(xvec, yvec, fz,col="white",
          xlab="Online Activities",ylab="Reading Attitude",zlab="Predicted Reading Skills",smooth=F)
}

#opens the graphic
curve_3d(yhat)
#saves the graphic as an external file
#snapshot3d("results_upi2.png")
#################################





























