library(lavaan)
library(mvtnorm)
library(scatterplot3d)
library(nlsem)

##################################################
# lms exercise 1 (demonstration)
##################################################

# this is the same data set as in the morning session for cpi
dat.kj <- read.table("../data/data_kj.dat",header=T)

####################################
# We do not need: to build product indicators
#                 to center the data
#                 to specify any residual covariances
####################################

head(dat.kj)

####################################
# the following script leads through the different parts of nlsem
####################################

# specify the general model
lms <- specify_sem(num.x=4, num.y=2, num.xi=2, num.eta=1, xi="x1-x2,x3-x4",
                   eta="y1-y2", interaction="xi1:xi2")
lms

lms$matrices
lms$info

# additional specifications: here set factor loadings to 1
# 1. extract the model
specs1 <- as.data.frame(lms)


# this it notation in nlsem
# Lambda: factor loading
# Gamma: Linear effect xi=>eta
# Beta: Linear effect eta=>eta
# Theta: Residual variance indicators
# Psi: Residual variance eta
# Phi: Variance predictors (does not include product terms; they don't exist for LMS)
# nu: intercepts of indicators
# alpha: intercept of eta
# tau: Means of xi
# Omega: Interaction and quadratic effects
specs1$label

# this is the organisation of matrices, e.g. for phi:
# 1 2
# 3 4
# or omega:
# 1 2
#   3
# so 1 and 3 are quadratic effects, and 2 is interaction

specs1
# number: fixed parameter (e.g., 0 or 1)
# NA: free parameter 

# lambda looks like this
matrix(specs1[specs1$label %in% paste0("Lambda.x", c(1:8)), "class1"],4,2,byrow=F)
# omega
matrix(specs1[specs1$label %in% paste0("Omega", c(1:4)), "class1"],2,2,byrow=F)
# theta
matrix(specs1[specs1$label %in% paste0("Theta.d", c(1:16)), "class1"],4,4,byrow=F)

#############################
# 2. specify a new model
#############################
# this sets the parameters nu.x and nu.y (intercepts) to zero
specs1[specs1$label %in% paste0("nu.x", 1:4), "class1"] <- 0
specs1[specs1$label %in% paste0("nu.y", 1:2), "class1"] <- 0
# latent means 
specs1[specs1$label %in% paste0("tau", 1:2), "class1"] <- 0

# set the factor loadings to 1
specs1[specs1$label %in% paste0("Lambda.x", c(2,8)), "class1"] <- 1
specs1[specs1$label %in% paste0("Lambda.y", 2), "class1"] <- 1

# 3. Recreate an object for nlsem 
lms2 <- create_sem(specs1)

# random start parameters
start <- runif(count_free_parameters(lms2))

# run LMS
res_lms <- em(lms2, dat.kj, start)
# convergence (plots the loglikelihood)
plot(res_lms)
# the model has converged

# parameters only
res_lms

# parameters and standard errors (sandwich estimator)
summary(res_lms)

# QML: alternative to LMS 
# (faster, but convergence is a bit sensitive to starting values)
res_qml <- qml(lms2, dat.kj, start)
# problem here: if starts produce a non-pos.def phi matrix

# better starting values
specs1$label[is.na(specs1$class1)]

start2 <- c(.3,.3,      # gamma
            rep(.25,6), # theta
            1,          # psi
            1,.5,1,     # phi
            0,   # alpha, kappa
            .2)         # omega
length(start2)==length(specs1$label[is.na(specs1$class1)])
res_qml <- qml(lms2, dat.kj, start2)

summary(res_qml)


##################################################
# Alternative way: specify it via lavaan syntax
##################################################
model.lms <- '
class: 1
xi1  =~ v1+1*v2
xi2  =~ s1+1*s2
eta  =~ c1+1*c2
eta ~ 1+ xi1 + xi2 + xi1:xi2
v1 ~ 0*1
v2 ~ 0*1
s1 ~ 0*1
s2 ~ 0*1
c1 ~ 0*1
c2 ~ 0*1
'

lms.lav <- lav2nlsem(model.lms, constraints = "direct1", class.spec = "class")
# results in an equivalent model to the one above
# note: ``class: 1'' is part of the syntax because the 
# package allows for several classes (see day 2)
##################################################





##################################################
##################################################
# lms exercise 2
# in the 2nd exercise you can build a model
# for the same data set as for the upi (exercise 3 in morning session)
# you can use both lavaan syntax or nlsem syntax (or either of them)
##################################################
##################################################

##################################################
# in the PISA data set 2009 (OECD, 2012)
##################################################
dat <- read.table("../data/pisa_online1.dat",header=T,na.strings = "999")
#delete missings: nlsem does not allow missing at the moment
dat <- na.omit(dat)

dim(dat)
# use a subset (otherwise nlsem runs very long)
dat <- dat[1:300,]

# non-centered data set 
dat2 <- dat

# centered data set
dat$on1 <- dat$on1-mean(dat$on1)
dat$on2 <- dat$on2-mean(dat$on2)
dat$on3 <- dat$on3-mean(dat$on3)

dat$att1 <- dat$att1-mean(dat$att1)
dat$att2 <- dat$att2-mean(dat$att2)
dat$att3 <- dat$att3-mean(dat$att3)

###########################
# model for centered data 
# use the code from above to start here. You can use the lavaan coding and 
# then transform it in the nlsem format

# NOTE: you need to set the means (tau) to zero by hand [BUG]
# if you used lavaan code (even if you set them to zero there)

###########################
# model for un-centered data but non-zero-means latent vars


###########################
# model for un-centered data but zero-means latent vars

# random start parameters

# run LMS with lower convergence criterion (speed up, just for the example)
# nlsem expects a sequence of x1.., y1.. in the data.frame!
res_lms1 <- em(lms1a, dat , start1a,convergence = .1)

# plot convergence of em algorithm

# investigate parameters



###########################
# model including quadratic effects

###########################
# comparison model without ...?


# run LMS with lower convergence criterion (speed up, just for the example)


# model difference test ``anova''
# NOTE: Provides wrong p-value [BUG]













